#include <math.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
using namespace std;

const int DIM = 6;

enum Casella {VUOTA, MINA, ESPLORATA_VUOTA, ESPLORATA_MINA};

struct CampoMinato {
  int mosse;
  Casella campo[DIM][DIM];
};

void inizializza(CampoMinato* pm, int num) {
  pm->mosse = 0;
  for(int i=0; i<DIM; i++) 
    for(int j=0; j<DIM; j++)
      pm->campo[i][j] = VUOTA;
  int p = 0;
  do{
    int a = rand()%(DIM*DIM);
    if(pm->campo[a/DIM][a%DIM] == VUOTA) {
      pm->campo[a/DIM][a%DIM] = MINA;
      p++; 
    }
  }while(p<num);
  /* La soluzione precedente funziona "bene" se num e' piccolo rispetto a DIM*DIM
     Se num non e' piccolo rispetto a DIM*DIM puo' succedere che per svariate volte
     si cerchi di piazzare una mina in una casella dove ce n'e' gia' un'altra. 
     Nel caso in cui num non e' piccolo rispetto a DIM*DIM e' preferibile una
     soluzione basata sulla seguente idea:
     - mettere all'interno di un array tutti i valori compresi tra 0 e DIM*DIM
     - mescolare l'array
     - prendere i primi num valori (che a questo punto sono tutti diversi)
     - usare questi num valori in modo analogo a quanto fatto nella soluzione corrente.
  */ 
}

int suggerimento(const CampoMinato* pm, int vett[]) {
  int q = 0;
  for(int i=0; i<DIM; i++) {
    bool trovato = false;
    for(int j=0; j<DIM; j++) {
      if(pm->campo[j][i] == MINA || pm->campo[j][i] == ESPLORATA_MINA) {
        trovato = true;
        break;
      }
    }
    if(trovato)
      vett[q++] = i;
  }
  return q;
}

bool caricaPartita(CampoMinato* pm, const char nomef[]) {
  fstream f;
  f.open(nomef, ios::in);
  if(!f) return false;
  f >> pm->mosse;
  for(int i=0; i<DIM; i++)
    for(int j=0; j<DIM; j++) {
      char c;
      f >> c;
      switch(c) {
        case 'v': 
          pm->campo[i][j] = VUOTA;
          break;
        case 'm':
          pm->campo[i][j] = MINA;
          break;
        case 'V': 
          pm->campo[i][j] = ESPLORATA_VUOTA;
          break;
        case 'M':
          pm->campo[i][j] = ESPLORATA_MINA;
          break;
      }
    }
  return true;
}

bool vicinoMina(const CampoMinato* pm, int x, int y) {
  int startx = x == 0 ? 0 : x - 1;
  int endx = x == DIM - 1 ? DIM - 1: x + 1;
  int starty = y == 0 ? 0 : y - 1;
  int endy = y == DIM - 1 ? DIM - 1: y + 1;
  for(int i=startx; i<=endx; i++)
    for(int j=starty; j<=endy; j++)
      if((i != x || j != y) && (pm->campo[i][j] == MINA || pm->campo[i][j] == ESPLORATA_MINA))
        return true;
  return false;
}

char converti(Casella c) {
  switch(c) {
    case VUOTA: return '#';
    case ESPLORATA_VUOTA: return ' ';
    case MINA: return '#';
    case ESPLORATA_MINA: return '*';
  }
}

void stampa(const CampoMinato* pm) {
  for(int i=0; i<DIM; i++) {
    for(int j=0; j<DIM; j++) 
      if(vicinoMina(pm, i, j) && pm->campo[i][j]==ESPLORATA_VUOTA)
        cout << '!';
      else 
        cout << converti(pm->campo[i][j]);
    cout << endl;
  }
}
